
# Created by Amin Rahmani, Ph.D., P.E. - February 3, 2018
# Copyright © 2018 Amin Rahmani. All rights reserved.

#========================================================================================
#                               ****   Notes  ****
# This code performs soil-pile interaction analysis in saturated soil condition.
# You need to makes changes on the lines that the comment "User-Defined" appears. Please go through all the command lines before running it.
# The code creates a half mesh. The model is symmetric in the direction perpendicular to shaking.
# Units are in Metric (kN, ton, sec, m). Check the units if you work with Imperial units. 
# The input motions should be available in two separate .txt files. Time in "SCTime.txt" and Displacement in "SCDisp.txt".
# When compliant boundary condition is used, the input motions should be available in two separate .txt files. Time in "Time.txt" and Velocity in "Velocity.txt".
# Outputs will be saved in a folder titled "Outputs".
#========================================================================================

wipe all


set startT [clock seconds]

##### This will create an input file for GID program if you want to check the final geometry.
set meshFile [open soilmesh.flavia.msh w]
##########################################################################################
	

#######################################################################
#### Input parameters used for creating the geometry of soil-pile model 
#######################################################################

set gravX  0.0000
set gravY  0.0000
set gravZ -9.8100				   ;# gravity in direction Z. The value would be -32.2 if Imperial units are used.

set BP       0.67                        ;# User-defined --- pile width                                         
set LP       16.5                        ;# User-defined --- pile length --- ** Note: Zone meshing in vertical direction should be consistent with the pile length.
						                 ;# zo and z1 defined below should be set in a way that LP >= z0+z1
set HZ       17.5                        ;# User-defined --- Model total height

#### You can apply 3 levels of mesh sizes in the horizontal direction, zone 0, 1, 2, and 3 represent areas from farthest to nearest to pile center.
set xSize0   3.0                           ;# User-Defined --- x direction element size - zone 0 
set ySize0   2.0                           ;# User-Defined --- y direction element size - zone 0
set xSize1   [expr $BP*2.0]                ;# x direction element size - zone 1
set ySize1   [expr $BP*2.0]                ;# y direction element size - zone 1
set xSize2   [expr $BP*1.0]                ;# x direction element size - zone 2
set ySize2   [expr $BP*1.0]                ;# y direction element size - zone 2
set xSize3   [expr $BP/2.0]                ;# x direction element size - zone 3 (within pile physical volume)
set ySize3   [expr $BP/2.0]                ;# y direction element size - zone 3 (within pile physical volume)

set WInf     0.4						   ;# User-Defined --- Thickness of a layer of soil ringing the pile representing interface element

# Total length and width of the model
set LX       [expr 2*($WInf+0.5*$BP+8*$BP+0.0)]                  ;# User defined --- Model lenght in direction X. Modify the number "0.0" in a way that LX match the length of your model.       
set BY       [expr 2*($WInf+0.5*$BP+8*$BP+0.0)]                   ;# User defined --- Model width in direction Y. Modify the number "0.0" in a way that BY match the width of your model.  

#### You can apply 3 levels of mesh sizes in the vertical direction, z0, z1, and z2 represent layer thichness at the top, middle, and bottom layers.
set z0 4.5   					;# User-Defined - total thickness of zone z0
set z1 12.0 					;# User-Defined - total thickness of zone z1
								; # zo and z1 should be set in a way that LP >= z0+z1
set z2 1.0 						;# User-Defined - total thickness of zone z2
 
set zSize0    1.5               ;# User-Defined - element thickness in zone z0
set zSize1    2.0               ;# User-Defined - element thickness in zone z1
set zSize2    1.0               ;# User-Defined - element thickness in zone z2

set numXele  [expr int(($LX-16*$BP-$BP-2*$WInf)/$xSize0 + 8*$BP/$xSize1 + 8*$BP/$xSize2 + $BP/$xSize3+2)   ]      ;# number of elements in x  direction 
set numYele  [expr int(($BY-16*$BP-$BP-2*$WInf)/$ySize0 + 8*$BP/$ySize1 + 8*$BP/$ySize2 + $BP/$ySize3+2)/2 ]      ;# number of elements in y  direction --- *** Note *** half mesh
set numZele  [expr int($z0/$zSize0+$z1/$zSize1+$z2/$zSize2)]                                                      ;# number of elements in z  direction    direction


###########################################################
#### Creating Nodes for 3-dimensional single column of soil
###########################################################

model basic -ndm 3 -ndf 4

#### For GID Program Use
puts $meshFile "MESH stdBrick dimension 3 ElemType Hexahedra Nnode 8"
puts $meshFile "Coordinates"
######################################################################

set numXnode  [expr $numXele+1]
set numYnode  [expr $numYele+1]
set numZnode  [expr $numZele+1]

set m 0
for {set i 1} {$i <= $numXnode} {incr i 1} {
  for {set j 1} {$j <= $numYnode} {incr j 1} {
    for {set k 1} {$k <= $numZnode} {incr k 1} {

      if { $i == $numXele/2 && $j == $numYele} {
           set xdim [expr $LX/2 - $BP/2 + $BP*(1-0.5*pow(2,0.5))/2]
           set ydim [expr $BY/2 - $BP/2 + $BP*(1-0.5*pow(2,0.5))/2]
      } else {
        if { $i == [expr $numXele/2+2] && $j == $numYele} {
           set xdim [expr $LX/2 + $BP*(0.5*pow(2,0.5))/2]
           set ydim [expr $BY/2 - $BP/2 + $BP*(1-0.5*pow(2,0.5))/2]
      } else {

      ### nodes xdim in zone 0
      if { $i <= [expr (($LX/2.0-8*$BP-$BP/2.0-$WInf )/$xSize0+1)] } {                                               
         set xdim [expr ($i-1)*$xSize0] 
      } else {
        if { $i > [expr $numXnode - ($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0 ] } {
           set xdim [expr $LX-($numXnode-$i)*$xSize0]
      ### nodes xdim in zone 1
      } else {
        if { $i > [expr (($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0+1)] && $i <= [expr (($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0 + (8*$BP)/(2*$xSize1) +1) ] } {
             set xdim [expr ($xSize1*($i-(($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0 + 1)) + $xSize0*($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0 ) ]             
      } else { 
        if { $i > [expr (($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0+(8*$BP)/(2*$xSize1) + (8*$BP)/($xSize2)+ $BP/$xSize3 +2+ 1)] && $i <= [expr $numXnode-(($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0) ] } { 
             set xdim [expr $LX/2.0 + $BP/2.0 + $WInf + 4*$BP + ($i-(($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0+ 8*$BP/(2*$xSize1)+(8*$BP)/$xSize2 + $BP/$xSize3+ 2 +1))*$xSize1]
      ### nodes xdim in zone 2 
      } else {
        if { $i > [expr (($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0+(8*$BP)/(2*$xSize1) + 1)] && $i <= [expr (($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0+(8*$BP)/(2*$xSize1) + (8*$BP)/(2*$xSize2) +1)] } {
             set xdim [expr ($LX/2.0- $BP/2.0-4*$BP-$WInf + ($i-(($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0+(8*$BP)/(2*$xSize1) +1))*$xSize2)]
      } else { 
        if { $i > [expr (($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0)+(8*$BP)/(2*$xSize1)+(8*$BP)/(2*$xSize2)+$BP/$xSize3+ 2 + 1] && $i <= [expr $numXnode-(($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0-4*$BP/$xSize1) ] } { 
             set xdim [expr ($LX/2.0+ $BP/2.0 +$WInf+ ( $i-(($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0+(8*$BP)/(2*$xSize1)+(8*$BP)/(2*$xSize2)+$BP/$xSize3+2 +1))*$xSize2)]
      ### nodes xdim in zone 3
      } else {
        if { $i == $numXele/2 } {
             set xdim [ expr $LX/2- $BP/2]
      } else {
        if  { $i == $numXele/2 +3 } {
             set xdim [ expr $LX/2+ $BP/2+$WInf]
      } else {

             set xdim   [expr ($LX/2.0-$BP/2.0+($i-(($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0+ 8*$BP/(2*$xSize1)+8*$BP/(2*$xSize2)+1+1))*$xSize3) ]   
             }        
      }
      }
      }
      }
      }
      }
      }

      ### nodes ydim in zone 0
      if { $j <= [expr (($BY/2.0-8*$BP-$BP/2.0-$WInf)/$ySize0+1)] } {                                               
         set ydim [expr ($j-1)*$ySize0] 
        
      ### nodes ydim in zone 1
      } else {
        if { $j > [expr (($BY/2.0-8*$BP-$BP/2.0-$WInf)/$ySize0+1)] && $j <= [expr (($BY/2.0-8*$BP-$BP/2.0-$WInf)/$ySize0 + (8*$BP)/(2*$ySize1) + 1) ] } {
             set ydim [expr ($ySize1*($j-(($BY/2.0-8*$BP-$BP/2.0-$WInf)/$ySize0 + 1)) + $ySize0*($BY/2.0-8*$BP-$BP/2.0-$WInf)/$ySize0 ) ]             

      ### nodes ydim in zone 2 
      } else {
        if { $j > [expr (($BY/2.0-8*$BP-$BP/2.0-$WInf)/$ySize0+(8*$BP)/(2*$ySize1) + 1)] && $j <= [expr (($BY/2.0-8*$BP-$BP/2.0-$WInf)/$ySize0+(8*$BP)/(2*$ySize1) + (8*$BP)/(2*$ySize2) +1)] } {
             set ydim [expr ($BY/2.0- $BP/2.0-$WInf-4*$BP + ($j-(($BY/2.0-8*$BP-$BP/2.0-$WInf)/$ySize0+(8*$BP)/(2*$ySize1) +1))*$ySize2)]
      ### nodes ydim in zone 3
      } else {
         if { $j == $numYele} {
            set ydim [ expr $BY/2- $BP/2]
      } else {
         if { $j == [expr $numYele+1] || $j == [expr $numYele+2] } {
             set ydim   [expr ($BY/2.0-$BP/2.0 + ($j-$numYele)*$ySize3) ]    
      } else {
         if { $j == [expr $numYele+3] } {
             set ydim [expr $BY/2.0+$BP/2.0+$WInf ]
      } else {
         if { $j > [expr $numYele+3] && $j <= [expr $numYele+1+1+$eleNum_2+1] } {
             set ydim [expr $BY/2.0+$BP/2.0+$WInf+($j-($numYele+3))*$ySize2 ]
      } else {
         if { $j > [expr $numYele+1+1+$eleNum_2+1] && $j <= [expr $numYele+1+1+$eleNum_2+$eleNum_1+1] } {
             set ydim [expr $BY/2.0+$BP/2.0+$WInf+4.0*$BP+($j-($numYele+1+1+$eleNum_2+1))*$ySize1 ]
      } else {
         if { $j > [expr $numYele+1+1+$eleNum_2+$eleNum_1+1] } {
             set ydim [expr $BY/2.0+$BP/2.0+$WInf+8.0*$BP+($j-($numYele+1+1+$eleNum_2+$eleNum_1+1))*$ySize0 ]
     
      }
      }}
      }}}}}}
}
}
      if { $k <= [expr ($z2/$zSize2+1)] } {
          set   zdim [expr ($k-1)*$zSize2]
      } else {
         if { $k > [expr ($z2/$zSize2)+($z1/$zSize1)+1] } {
          set   zdim [expr $z2+$z1+($k-(($z2/$zSize2)+($z1/$zSize1)+1))*$zSize0]
      } else {
          set   zdim [expr $z2+($k-(($z2/$zSize2)+1))*$zSize1]
      }
      }
      set   nodeNum   [expr $m*$numZnode+$k] 
      node  $nodeNum  $xdim $ydim $zdim
   
	  #### For GID Program Use
      puts $meshFile "$nodeNum  $xdim $ydim $zdim"
	  #############################################
	  
###### Specifying Node numbers in center and edge of pile

if { $i == [expr $numXele/2] && $j == [ expr $numYele ] && $k == [ expr int(($HZ-$LP)/$zSize2+1) ] } {
    set p1 $nodeNum
    puts "p1=$p1"
   } else {
   if { $i == [expr $numXele/2+1] && $j == [ expr $numYele ] && $k == [ expr int(($HZ-$LP)/$zSize2+1) ] } {
    set p2 $nodeNum
    puts "p2=$p2"
   } else {
   if { $i == [expr $numXele/2+2] && $j == [ expr $numYele ] && $k == [ expr int(($HZ-$LP)/$zSize2+1) ] } {
    set p3 $nodeNum
    puts "p3=$p3"
   }
   } 
   }
   }
    set m [expr ($m+1)]
  } 
}

#### For GID Program Use
puts $meshFile "end coordinates"
#################################

##########################################
#### Define Soil Materials
##########################################

# User-Defined --- define behavior of soil at different layers

##layer 1
set rhof    1.0
set cohes1  2.5
set Gs1     [expr 200.0*$cohes1]
set nou1    0.4
###set Es1     [expr 9.0*$Bs1*$Gs1/(3.0*$Bs1+$Gs1)]
set Bs1            [expr 2.0*$Gs1*(1+$nou1)/3.0/(1-2.0*$nou1)]
set rhos1   1.8
set peakShearStra 0.1
set porosity1 0.6
set Perm1 2.0e-3;   #Soil Permeability
nDMaterial PressureIndependMultiYield 1 3 $rhos1  $Gs1 $Bs1 $cohes1 $peakShearStra

##layer 2
set cohes2  6.5
set nou2    0.4
set Gs2      [expr 200.0*$cohes2]
set Bs2       [expr 2.0*$Gs2*(1+$nou2)/3.0/(1-2.0*$nou2)]
set rhos2  1.8
set peakShearStra 0.1
set porosity2 0.6
set Perm2 2.0e-3;   #Soil Permeability
nDMaterial PressureIndependMultiYield 2 3 $rhos2  $Gs2 $Bs2 $cohes2 $peakShearStra

##layer 3
set cohes3  9.0
set nou3    0.4
set Gs3      [expr 200.0*$cohes3]
set Bs3       [expr 2.0*$Gs3*(1+$nou3)/3.0/(1-2.0*$nou3)]
set rhos3   1.8
set peakShearStra 0.1
set porosity3 0.6
set Perm3 2.0e-3;   #Soil Permeability
nDMaterial PressureIndependMultiYield 3 3 $rhos3  $Gs3 $Bs3 $cohes3 $peakShearStra

##layer 4
set cohes4  12.0
set nou4    0.4
set Gs4      [expr 200.0*$cohes4]
set Bs4       [expr 2.0*$Gs4*(1+$nou4)/3.0/(1-2.0*$nou4)]
set rhos4   1.8
set peakShearStra 0.1
set porosity4 0.6
set Perm4 2.0e-3;   #Soil Permeability
nDMaterial PressureIndependMultiYield 4 3 $rhos4  $Gs4 $Bs4 $cohes4 $peakShearStra


set G5            41460.0
set nou5           0.3
set B5            [expr 2.0*$G5*(1+$nou5)/3.0/(1-2.0*$nou5)]
##set Es1           [expr 9.0*$B1*$G1/(3.0*$B1+$G1)]
set rhos5          2.0
set Phi5           39.50 
set refPress       80.0
set pressDependCoe 0.5
set PTAng          27.0
set contrac        0.05
set dilat1         0.60
set dilat2         3.0
set liquefac1      0.0
set liquefac2      0.0
set liquefac3      0.0
set peakShearStra 0.1
set porosity5 0.6
set Perm5 2.0e-3;   #Soil Permeability
nDMaterial PressureDependMultiYield 5 3 $rhos1  $G5 $B5 $Phi5 $peakShearStra $refPress $pressDependCoe $PTAng $contrac $dilat1 $dilat2 $liquefac1 $liquefac2 $liquefac3 20 0.595

set bulk 2.2e9 ; #N/m^2 #User-Defined --- pore water bulk modulus

##########################################
#### Define Soil Elements
##########################################

#### For GID Program Use
puts $meshFile "Elements"
#################################

set m 0

for {set i 1} {$i <= $numXele } {incr i 1} {
  for {set j 1} {$j <= $numYele } {incr j 1} {
    for {set k 1} {$k <= $numZele} {incr k 1} {
      
      set eleNum [expr $m*$numZele+$k] 
      set n1  [expr ($i-1)*($numYnode*$numZnode)+$k+($j-1)*$numZnode] 
      set n2  [expr $n1+($numYnode*$numZnode)] 
      set n3  [expr $n2+$numZnode] 
      set n4  [expr $n1+$numZnode] 
      set n5  [expr $n1+1]
      set n6  [expr $n2+1]
      set n7  [expr $n3+1]
      set n8  [expr $n4+1]
      
	 # User-Defined --- Depending on the number of soil layers, you may need more "if-else" command lines. Remember to change "$K <= 5". It should represent soil stratigraphy.
	 # At this stage, permeability must be set to 1000.0 to avoid pore water pressure generation during geostatic analysis. 
      
	  if { $k >= 9 } {  ;# User-Defined
		element brickUP $eleNum $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8 1  [expr $bulk/$porosity1] 1.938 1000.0 1000.0 1000.0 $gravX $gravY [expr $gravZ]	  
		puts $meshFile "$eleNum $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8 1"
      } else { 
	  if { $k >= 6 && $k <= 8 } { ;# User-Defined 
        element brickUP $eleNum $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8 2  [expr $bulk/$porosity2] 1.938 1000.0 1000.0 1000.0 $gravX $gravY [expr $gravZ]	
		puts $meshFile "$eleNum $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8 2"
	  } else { 
	  if { $k >= 3 && $k <= 5 } { ;# User-Defined
	    element brickUP $eleNum $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8 3  [expr $bulk/$porosity3] 1.938 1000.0 1000.0 1000.0 $gravX $gravY [expr $gravZ]	
		puts $meshFile "$eleNum $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8 3"
	  } else { 
	  if { $k == 2 } { ;# User-Defined
	    element brickUP $eleNum $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8 4  [expr $bulk/$porosity4] 1.938 1000.0 1000.0 1000.0 $gravX $gravY [expr $gravZ]	
		puts $meshFile "$eleNum $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8 4"   
	  } else { ;# User-Defined
	    element brickUP $eleNum $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8 5  [expr $bulk/$porosity5] 1.938 1000.0 1000.0 1000.0 $gravX $gravY [expr $gravZ]
		puts $meshFile "$eleNum $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8 5"
	  }}}}	   	  	   

    }
    set m [expr ($m+1)]
  }
}

##############################################
#### Prints Total Number of Nodes and Elements
##############################################
puts "Number of Nodes    =  $nodeNum"
puts "Number of Elements =  $eleNum " 

#### For GID Program Use
puts $meshFile "end elements"
#############################


##############################################
#### Boundary conditions
##############################################

# Symmetry Plane
fixY [expr $BY/2.0] 0 1 0 

# Bottom Boundary 
fixZ  0.0  1 1 1 0   -tol 1e-12

# User-defined --- Ground Water Depth With Respect to Model Base 
fixZ [expr $z0+$z1+$z2] 0 0 0 1 ; #User-Defined - Use fixZ ... 0 0 0 1 for all levels above ground water table.

# Lateral Boundaries 
for {set k 2 } { $k <= $numZnode } { incr k 1 } {
  for {set j 2} {$j <=  $numYnode } { incr j 1 } {

      equalDOF $k [expr ($numXnode-1)*($numZnode*$numYnode)+$k+(($j-2)*$numZnode)]  1  2  3      ;# right side yz boundary 
      equalDOF $k [expr ($j-1)*$numZnode+$k]  1   2  3                                           ;# left  side yz boundary                                                                                
        }

      equalDOF $k  [expr int($numXnode*$numZnode*$numYnode-$numZnode+$k)]  1 2 3

  for {set i 2} {$i <=  [expr ($numXnode-1)]} {incr i 1} {
     equalDOF $k [expr ($i-1)*($numZnode*$numYnode)+$k]      1    2  3                                 ;# right side xz boundary
     equalDOF $k [expr $numZnode*($numYnode-1)+($i-1)*($numZnode*$numYnode)+$k]      1    2  3         ;# right side xz boundary
       }
}

puts " ## Soil is built!! ## "


##############################################
#### Elastic Geostatic Analysis
##############################################

numberer Plain
system SparseSYM
#system Mumps
test NormDispIncr 1.0e-3 100 1
algorithm KrylovNewton
constraints Penalty 1.e18 1.e18
integrator Newmark 0.5 0.25
analysis Transient 

 for {set numIncr 1} {$numIncr <= 5 } {incr numIncr 1} {
       puts "##### Elastic Geostatic Analysis step : $numIncr #####";
       analyze 1 10.0
     }

##############################################
#### Switch into Plastic State 
##############################################

# User-Defined
 updateMaterialStage -material 1 -stage 1	; #User-Defined --- Must be modified per the number of material types and tags.
 updateMaterialStage -material 2 -stage 1	; #User-Defined --- Must be modified per the number of material types and tags.
 updateMaterialStage -material 3 -stage 1	; #User-Defined --- Must be modified per the number of material types and tags.
 updateMaterialStage -material 4 -stage 1	; #User-Defined --- Must be modified per the number of material types and tags.
 updateMaterialStage -material 5 -stage 1	; #User-Defined --- Must be modified per the number of material types and tags.

for {set numIncr 1} {$numIncr <= 5 } {incr numIncr 1} {
      puts "##### Plastic Geostatic Analysis step : $numIncr #####";
      analyze 1 0.01
    }

##############################################
#### Update Stiffness of Materials 
##############################################	
updateMaterials -material 1 shearModulus [expr $Gs1] ; # User-Defined --- Change Mat tag and soil parameter tag per the prescribed tags
updateMaterials -material 1 bulkModulus  [expr $Bs1] ; # User-Defined --- Change Mat tag and soil parameter tag per the prescribed tags

updateMaterials -material 2 shearModulus [expr $Gs2] ; # User-Defined --- Change Mat tag and soil parameter tag per the prescribed tags
updateMaterials -material 2 bulkModulus  [expr $Bs2] ; # User-Defined --- Change Mat tag and soil parameter tag per the prescribed tags

updateMaterials -material 3 shearModulus [expr $Gs3] ; # User-Defined --- Change Mat tag and soil parameter tag per the prescribed tags
updateMaterials -material 3 bulkModulus  [expr $Bs3] ; # User-Defined --- Change Mat tag and soil parameter tag per the prescribed tags

updateMaterials -material 4 shearModulus [expr $Gs4] ; # User-Defined --- Change Mat tag and soil parameter tag per the prescribed tags
updateMaterials -material 4 bulkModulus  [expr $Bs4] ; # User-Defined --- Change Mat tag and soil parameter tag per the prescribed tags

updateMaterials -material 5 shearModulus [expr $Gs5] ; # User-Defined --- Change Mat tag and soil parameter tag per the prescribed tags
updateMaterials -material 5 bulkModulus  [expr $Bs5] ; # User-Defined --- Change Mat tag and soil parameter tag per the prescribed tags


##############################################
#### Introduce Pile into the Numerical Model
##############################################

loadConst -time 0.0
wipeAnalysis

##############################################
#### Define Pile Nodes
##############################################

model basic -ndm 3 -ndf 6

#### For GID Program Use
puts $meshFile "MESH beamcolumn dimension 3 ElemType linear Nnode 2"
puts $meshFile "Coordinates"
#######################################################################

set m 0
set n $p1
for {set i 1} {$i <= 3} {incr i 1} {
  if { $i == 2 } { set n $p2 }
  if { $i == 3 } { set n $p3 }
  for {set j 1} {$j <= 2} {incr j 1} {
    for {set k 1} {$k <= [expr int($z0/$zSize0+$z1/$zSize1+($LP-$z0-$z1)/$zSize2+1)]} {incr k 1} {

      if { $i == 1 && $j == 1} {
           set xdim [expr $LX/2 - $BP/2 + $BP*(1-0.5*pow(2,0.5))/2]
           set ydim [expr $BY/2 - $BP/2 + $BP*(1-0.5*pow(2,0.5))/2]
      } else {
        if { $i == 3 && $j == 1} {
           set xdim [expr $LX/2 + $BP*(0.5*pow(2,0.5))/2]
           set ydim [expr $BY/2 - $BP/2 + $BP*(1-0.5*pow(2,0.5))/2]
      } else {

      set xdim [expr ($i-1)*$xSize3 + $LX/2.0 - $BP/2.0]
      set ydim [expr ($j-1)*$ySize3 + $BY/2.0 - $BP/2.0]
      }
      }
	  
      if { $k <= [expr ($LP-$z0-$z1)/$zSize2+1] } {
          set   zdim [expr ($k-1)*$zSize2 + $HZ   -   $LP]
      } else {
        if { $k > [expr ($LP-$z0-$z1)/$zSize2+($z1/$zSize1)+1] } {
          set   zdim [expr $z2+$z1+($k-(($LP-$z0-$z1)/$zSize2+($z1/$zSize1)+1))*$zSize0]
      } else {
          set   zdim [expr $z2+($k-(($LP-$z0-$z1)/$zSize2+1))*$zSize1]
      }
      }

      set nodeNum [expr int($m*($z0/$zSize0+$z1/$zSize1+($LP-$z0-$z1)/$zSize2+1)+$k+$numXnode*$numYnode*$numZnode)]
      node $nodeNum $xdim $ydim $zdim 
      
	  #### For GID Program Use
      puts $meshFile "$nodeNum $xdim $ydim $zdim"
	  ###########################################
	  
      if { $i == 2 && $j == 2 && $k == 1 } {
      set np $nodeNum    
	  
      }

		###	 Connecting pile nodes to soil nodes  
		#  Slaving Pile Node(only the lowest node) and its Edge Nodes to Soil Nodes
       if { $i != 2 || $j != 2 } {
          equalDOF  [expr $n+$k-1] $nodeNum 1 2 3 
         } 
       if { $i == 2 && $j == 2 && $k == 1 } {
          equalDOF  [expr $n+$k-1] $nodeNum 1 2 3 
         }  

    }     
    set m [expr ($m+1)]
    set n [expr ($n+$numZnode)]
  }
}

# Creating nodes for the horizontal rigid beams connecting pile main nodes to surrounding soil nodes.
for { set r 1 } { $r <= [expr int($z0/$zSize0+$z1/$zSize1+($LP-$z0-$z1)/$zSize2+1)] } { incr r 1 } {

      set dummyNode [expr 49999+$r]

      if { $r <= [expr ($LP-$z0-$z1)/$zSize2+1] } {
          set   zdim [expr ($r-1)*$zSize2 + $HZ   -   $LP]
      } else {
        if { $r > [expr ($LP-$z0-$z1)/$zSize2+($z1/$zSize1)+1] } {
          set   zdim [expr $z2+$z1+($r-(($LP-$z0-$z1)/$zSize2+($z1/$zSize1)+1))*$zSize0]
      } else {
          set   zdim [expr $z2+($r-(($LP-$z0-$z1)/$zSize2+1))*$zSize1]
      }
      }
    node  $dummyNode [ expr $LX/2 ] [ expr $BY/2 ]  $zdim
	
	#### For GID Program Use
    puts $meshFile "$dummyNode [ expr $LX/2 ] [ expr $BY/2 ]  $zdim"
    ################################################################

    equalDOF  $dummyNode  [expr $np+$r-1] 1 2 3 
	
	#### For GID Program Use
    puts "$dummyNode [ expr $LX/2 ] [ expr $BY/2 ]  $zdim"
	########################################################
    } 

# User - Defined ---- Pile nodes above ground surface
node 60000 [expr $LX/2.0] [expr $BY/2.0] [expr $HZ+1.0] 	; # User-Defined 
node 60001 [expr $LX/2.0] [expr $BY/2.0] [expr $HZ+2.0]		; # User-Defined
node 60002 [expr $LX/2.0] [expr $BY/2.0] [expr $HZ+3.0] 	; # User-Defined
node 60003 [expr $LX/2.0] [expr $BY/2.0] [expr $HZ+3.81]	; # User-Defined

#### For GID Program Use
puts $meshFile "60000 [expr $LX/2.0] [expr $BY/2.0] [expr $HZ+1.0]"  ; # User-Defined 
puts $meshFile "60001 [expr $LX/2.0] [expr $BY/2.0] [expr $HZ+2.0]"  ; # User-Defined 
puts $meshFile "60002 [expr $LX/2.0] [expr $BY/2.0] [expr $HZ+3.0]"  ; # User-Defined 
puts $meshFile "60003 [expr $LX/2.0] [expr $BY/2.0] [expr $HZ+3.81]" ; # User-Defined 
puts $meshFile "end coordinates"
####################################################################


##############################################
#### Pile Nodes Boundary Conditions
##############################################

 for { set i $np } { $i <= [expr int($np+($z0/$zSize0+$z1/$zSize1+($LP-$z0-$z1)/$zSize2))]} { incr i 1 } {
   fix $i 0 0 0 0 0 1
   } 

fix 60000 0 0 0 0 0 1
fix 60001 0 0 0 0 0 1
fix 60002 0 0 0 0 0 1
fix 60003 0 0 0 0 0 1

#######################################################
#### Define Pile Cross-section
#######################################################

# User-Defined Parameters

set D    0.67   ; # User-Defined --- Pile Diameter
set t    0.019	; # User-Defined --- Pile Wall Thickness
set A    [expr (3.1416*$D*$D/4.0-3.1416*($D-2*$t)*($D-2*$t)/4.0)] ; # User-Defined --- Pile Corss-sectional Area
set E    200.0e+6  ; # User-Defined --- Pile Young's Modulus
set Fy   69.0e+4   ; # User-Defined --- Pile Yeild Strength
set G    [expr $E/2.0/(1+0.30)] 
set Iz   [expr 3.1416*0.013*(pow(($D/2+($D-2*$t)/2)/2.0 , 3))] ; # User-Defined --- Pile area moment of inertia 
set Iy   [expr 3.1416*0.013*(pow(($D/2+($D-2*$t)/2)/2.0 , 3))] ; # User-Defined --- Pile area moment of inertia 
set J    [expr $Iz+$Iy]
set rhop 7.9 ; # User-Defined --- Pile mass density
set mass_pile [expr $rhop*$A]


#######################################################
#### Define Pile Material Behavior
#######################################################

uniaxialMaterial Elastic 7000 $E  						; #User Defined
uniaxialMaterial Steel01 7001 $Fy $E 0.0			    ; #User Defined
uniaxialMaterial Steel02 7002 $Fy $E 0.0 18 0.925 0.15  ; #User Defined

section Fiber 1 { 
  patch circ  7000 10 10 0.0 0.0 [expr $D/2-$t] [expr $D/2] 0.0 360.0
  }
section Elastic 3 $E $A $Iz $Iy $G $J

#### For GID Program Use
puts $meshFile "Elements"
#########################

#######################################################
#### Define Pile Elements
#######################################################

geomTransf PDelta 101 1 0 0
geomTransf Linear 102 0 0 1

set ne1 $np
set numIntgrPts 5

for { set k 1 } { $k <= [ expr $z0/$zSize0+$z1/$zSize1+($LP-$z0-$z1)/$zSize2] } { incr k 1 } {

  element dispBeamColumn [expr $eleNum+$k] $ne1 [expr $ne1+1]   $numIntgrPts 3  101 -mass $mass_pile   
  puts $meshFile "[expr $eleNum+$k] $ne1 [expr $ne1+1] 10"
  set ne1 [expr $ne1+1]

    }

### Pile Elements Above Ground Surface 

element dispBeamColumn 70000 $ne1  60000   $numIntgrPts 3  101 -mass $mass_pile	; #User-defined
element dispBeamColumn 70001 60000 60001   $numIntgrPts 3  101 -mass $mass_pile	; #User-defined
element dispBeamColumn 70002 60001 60002   $numIntgrPts 3  101 -mass $mass_pile	; #User-defined
element dispBeamColumn 70003 60002 60003   $numIntgrPts 3  101 -mass $mass_pile	; #User-defined

#### For GID Program Use
  puts $meshFile "70000 $ne1  60000 10" ; #User-defined
  puts $meshFile "70001 60000 60001 10"	; #User-defined
  puts $meshFile "70002 60001 60002 10"	; #User-defined
  puts $meshFile "70003 60002 60003 10"	; #User-defined
#######################################


#########################################################################
#### Define Rigid Elements to Connect Pile Node to Surrounding Soil Nodes
#########################################################################

#### Note: The elemnet has to be rigid
set Acon    [expr (3.1416*$D*$D/4.0)]
set Econ    200.0e+6 
set Fy   	69.0e+4
set Gcon   [expr $Econ/2.0/(1+0.30)]
set Izcon   [expr 1000.0*0.25*3.1416*(pow([expr $D/2.] , 4))]
set Iycon   [expr 1000.0*0.25*3.1416*(pow([expr $D/2.] , 4))]
set Jcon    [expr $Izcon+$Iycon]

set ne6 50000
set ne5 $np
set m [expr int($eleNum+$z0/$zSize0+$z1/$zSize1+($LP-$z0-$z1)/$zSize2)]
set rec_connectionele $m
set pile_elenum [expr int($z0/$zSize0+$z1/$zSize1+($LP-$z0-$z1)/$zSize2)]

for { set i 1 } { $i <=  [expr int($z0/$zSize0+$z1/$zSize1+($LP-$z0-$z1)/$zSize2+1)]  } { incr i 1 } {

    element elasticBeamColumn    [expr $m+1]   $ne6  [expr int($ne5 - ($pile_elenum+1)*3)]   $Acon $Econ $Gcon $Jcon $Iycon $Izcon 102
    element elasticBeamColumn    [expr $m+2]   $ne6  [expr int($ne5 - ($pile_elenum+1)*2)]   $Acon $Econ $Gcon $Jcon $Iycon $Izcon 102
    element elasticBeamColumn    [expr $m+3]   $ne6  [expr int($ne5 - ($pile_elenum+1)*1)]   $Acon $Econ $Gcon $Jcon $Iycon $Izcon 102
    element elasticBeamColumn    [expr $m+4]   $ne6  [expr int($ne5 + ($pile_elenum+1)*1)]   $Acon $Econ $Gcon $Jcon $Iycon $Izcon 102
    element elasticBeamColumn    [expr $m+5]   $ne6  [expr int($ne5 + ($pile_elenum+1)*2)]   $Acon $Econ $Gcon $Jcon $Iycon $Izcon 102
 
	#### For GID Program Use   
    puts $meshFile "[expr $m+1]   $ne6  [expr int($ne5 - ($pile_elenum+1)*3)] 10"
    puts $meshFile "[expr $m+2]   $ne6  [expr int($ne5 - ($pile_elenum+1)*2)] 10"
    puts $meshFile "[expr $m+3]   $ne6  [expr int($ne5 - ($pile_elenum+1)*1)] 10"
    puts $meshFile "[expr $m+4]   $ne6  [expr int($ne5 + ($pile_elenum+1)*1)] 10"
    puts $meshFile "[expr $m+5]   $ne6  [expr int($ne5 + ($pile_elenum+1)*2)] 10"
	#############################################################################
	
    set ne6 [expr $ne6+1]
    set ne5 [expr $ne5+1]
    set m   [expr $m+5]

    }      
     
#### For GID Program Use 	 
puts $meshFile "end elements"
close $meshFile
##############################

#########################################################################
#### Define Pile Head Loading Condition
#########################################################################

set headload 482.0 ; # User-Defined ---- Load at Pile Head
mass 60003 [expr -$headload/$gravZ] [expr -$headload/$gravZ ] [expr -$headload/$gravZ ] 0.0 0.0 0.0; #User-Defined --- Pile head node number needs to be modified. 

pattern Plain 100 "Linear" {

 load 60003 0.0 0.0 [expr -$headload] 0.0 0.0 0.0; # User-Defined --- Pile head node number and the direction of loading needs to be modified.

 }

#########################################################################
#### Analysis of Model Under Pile Loads
#########################################################################

numberer Plain
system SparseSYM
#system SparseGeneral
#system Mumps
test NormDispIncr 1.0e-2 100 1
algorithm KrylovNewton
constraints Penalty 1.e18 1.e18
integrator Newmark 0.5 0.25
analysis Transient 

for {set numIncr 1} {$numIncr <= 10 } {incr numIncr 1} {
       puts "##### Pile Load Static Analysis step : $numIncr #####";
       analyze 1 0.01
    }

puts " ## Pile Load was applied !! ## "

##############################################
#### Preparing Model for Seismic Analysis
##############################################

wipeAnalysis
remove recorders
loadConst -time 0.0

######################################################
#### Updating Soil Permeability for Seismic Shaking
######################################################

#set parameter tags 
for {set i 1} {$i <= [expr $numXele*$numYele*$numZele] } {incr i 1} {
    parameter [expr 100000+$i] element $i hPerm 
    parameter [expr 200000+$i] element $i vPerm 
    }

# User-Defined --- "k" values and updated permeability values should be modified to represent soil stratigraphy. It should be the same as layer numbers used in defining material to soil elements.
for {set i 1} {$i <= $numXele } {incr i 1} {
	for {set j 1} {$j <= $numYele } {incr j 1} {
	
		for {set k 1} {$k <= 1 } {incr k 1} {  ; # User-Defined
			updateParameter [expr 100000+$k+($j-1)*$numZele+($i-1)*$numZele*$numYele] $Perm5 ; #User-Defined --- Perm5
			updateParameter [expr 200000+$k+($j-1)*$numZele+($i-1)*$numZele*$numYele] $Perm5 ; #User-Defined --- Perm5
		}
		
		for {set k 2} {$k <= 2 } {incr k 1} {  ; # User-Defined
			updateParameter [expr 100000+$k+($j-1)*$numZele+($i-1)*$numZele*$numYele] $Perm4 ; #User-Defined --- Perm4 
			updateParameter [expr 200000+$k+($j-1)*$numZele+($i-1)*$numZele*$numYele] $Perm4 ; #User-Defined --- Perm4
		}
		
		for {set k 3} {$k <= 5 } {incr k 1} {  ; # User-Defined
			updateParameter [expr 100000+$k+($j-1)*$numZele+($i-1)*$numZele*$numYele] $Perm3 ; #User-Defined --- Perm3 
			updateParameter [expr 200000+$k+($j-1)*$numZele+($i-1)*$numZele*$numYele] $Perm3 ; #User-Defined --- Perm3
		}
		for {set k 6} {$k <= 8 } {incr k 1} {  ; # User-Defined
			updateParameter [expr 100000+$k+($j-1)*$numZele+($i-1)*$numZele*$numYele] $Perm2 ; #User-Defined --- Perm2 
			updateParameter [expr 200000+$k+($j-1)*$numZele+($i-1)*$numZele*$numYele] $Perm2 ; #User-Defined --- Perm2
		}
		for {set k 9} {$k <= $numZele } {incr k 1} {  ; # User-Defined
			updateParameter [expr 100000+$k+($j-1)*$numZele+($i-1)*$numZele*$numYele] $Perm1 ; #User-Defined --- Perm1 
			updateParameter [expr 200000+$k+($j-1)*$numZele+($i-1)*$numZele*$numYele] $Perm1 ; #User-Defined --- Perm1
		}			
		
	}
}
	
##############################################
#### User-Defined Rayleigh Damping
##############################################

set zeta 0.05;		# User-Defined --- Damping Ratio in Decimal
set w1  21.12;		# User-Defined --- Angular Frequency at Frequency 1
set w2  14.96;		# User-Defined --- Angular Frequency at Frequency 2

set a0 [expr $zeta*2.0*$w1*$w2/($w1 + $w2)];	# mass damping coefficient based on first and second modes
set a1 [expr $zeta*2.0/($w1 + $w2)];			# stiffness damping coefficient based on first and second modes

rayleigh $a0 0.0 $a1 0.0; 


######################################################################
#### Removing Constraints at Model Base in the Direction of Excitation
######################################################################

for {set i 1} {$i <= $numXnode} {incr i 1} {
  for {set j 1} {$j <= $numYnode} {incr j 1} {

        remove sp [expr 1+($i-1)*$numYnode*$numZnode+($j-1)*$numZnode] 1  

      }
    } 

########################################################
#### Creating Output Files in a Folder titled "Outputs"
########################################################

set dataDir Outputs
file mkdir $dataDir

eval "recorder Node    -file $dataDir/Accelpile.out       -time    -nodeRange $np [expr int($np+($z0/$zSize0+$z1/$zSize1+($LP-$z0-$z1)/$zSize2))]   -dof 1 2 3 accel"
eval "recorder Node    -file $dataDir/Accelpileabove.out  -time    -nodeRange 60000 60003   -dof 1 2 3 accel" ; #User-Difined ---- Pile node numbers above ground surface needs to be modified.

eval "recorder Node    -file $dataDir/disppile_con.out       -time    -nodeRange $np [expr int($np+($z0/$zSize0+$z1/$zSize1+($LP-$z0-$z1)/$zSize2))]   -dof 1 2 3 4 5 6 disp"
eval "recorder Node    -file $dataDir/disppileabove_con.out  -time    -nodeRange 60000 60003   -dof 1 2 3 4 5 6 disp" ; #User-Difined ---- Pile node numbers above ground surface needs to be modified.

eval "recorder Element -file $dataDir/pileforcesGlob_con.out           -time    -eleRange  [expr $eleNum+1] [expr int($eleNum+($z0/$zSize0+$z1/$zSize1+($LP-$z0-$z1)/$zSize2))]     globalForce"
eval "recorder Element -file $dataDir/pileforcesGlobabove_con.out      -time    -eleRange  69999 70003     globalForce"  ; #User-Difined ---- Pile element numbers above ground surface needs to be modified.

eval "recorder Node    -file $dataDir/PWP_LeftSide.out   -time    -nodeRange [expr int(($numXele/2-4)*$numYnode*$numZnode+1)] [expr int(($numXele/2-4)*$numYnode*$numZnode+1+$numZele)]   -dof 4   vel" ; #User-Difined ---- PWP about 4B at left side of the pile.
eval "recorder Node    -file $dataDir/PWP_RightSide.out  -time    -nodeRange [expr int(($numXele/2+4)*$numYnode*$numZnode+1)] [expr int(($numXele/2+4)*$numYnode*$numZnode+1+$numZele)]   -dof 4   vel" ; #User-Difined ---- PWP about 4B at right side of the pile.


################################################################################################
#### Rigid Base Boundary Condition - Apply Ground Motion At the Base in the Form of Displacement
################################################################################################

# set FACT  1
# set Gaccel 500

# timeSeries Path $Gaccel -fileTime SCTime.txt -filePath SCDisp.txt  -factor $FACT

# pattern Plain 5000 500 {

# for {set i 1} {$i <= $numXnode} {incr i 1} {
  # for {set j 1} {$j <= $numYnode} {incr j 1} {

        # sp [expr 1+($i-1)*$numYnode*$numZnode+($j-1)*$numZnode] 1 1.0 
		
      # }
    # }  
# }
#### Or you can use the code below to apply the motion. Both give the same results.
##pattern MultipleSupport 4000 {
##                                                                                
##groundMotion 2000 Plain -disp $Gaccel
##
##for {set i 1} {$i <= $numXnode} {incr i 1} {
##  for {set j 1} {$j <= $numYnode} {incr j 1} {
##
##        imposedMotion [expr 1+($i-1)*$numYnode*$numZnode+($j-1)*$numZnode] 1 2000 
##      }
##    }   
##}
########################################################################
#### End of Rigid Base Boundary Condition
########################################################################


########################################################################
#### Compliant Base Boundary Condition - Motion in the form of velocity
########################################################################

model basic -ndm 3 -ndf 3

set ID   200000
set ID_2 400000
set m 0
for {set i 1} {$i <= $numXnode} {incr i 1} {
  for {set j 1} {$j <= $numYnode} {incr j 1} {
    for {set k 1} {$k <= 1} {incr k 1} {

      if { $i == $numXele/2 && $j == $numYele} {
           set xdim [expr $LX/2 - $BP/2 + $BP*(1-0.5*pow(2,0.5))/2]
           set ydim [expr $BY/2 - $BP/2 + $BP*(1-0.5*pow(2,0.5))/2]
      } else {
        if { $i == [expr $numXele/2+2] && $j == $numYele} {
           set xdim [expr $LX/2 + $BP*(0.5*pow(2,0.5))/2]
           set ydim [expr $BY/2 - $BP/2 + $BP*(1-0.5*pow(2,0.5))/2]
      } else {

      ### nodes xdim in zone 0
      if { $i <= [expr (($LX/2.0-8*$BP-$BP/2.0-$WInf )/$xSize0+1)] } {                                               
         set xdim [expr ($i-1)*$xSize0] 
      } else {
        if { $i > [expr $numXnode - ($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0 ] } {
           set xdim [expr $LX-($numXnode-$i)*$xSize0]
      ### nodes xdim in zone 1
      } else {
        if { $i > [expr (($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0+1)] && $i <= [expr (($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0 + (8*$BP)/(2*$xSize1) +1) ] } {
             set xdim [expr ($xSize1*($i-(($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0 + 1)) + $xSize0*($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0 ) ]             
      } else { 
        if { $i > [expr (($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0+(8*$BP)/(2*$xSize1) + (8*$BP)/($xSize2)+ $BP/$xSize3 +2+ 1)] && $i <= [expr $numXnode-(($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0) ] } { 
             set xdim [expr $LX/2.0 + $BP/2.0 + $WInf + 4*$BP + ($i-(($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0+ 8*$BP/(2*$xSize1)+(8*$BP)/$xSize2 + $BP/$xSize3+ 2 +1))*$xSize1]
      ### nodes xdim in zone 2 
      } else {
        if { $i > [expr (($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0+(8*$BP)/(2*$xSize1) + 1)] && $i <= [expr (($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0+(8*$BP)/(2*$xSize1) + (8*$BP)/(2*$xSize2) +1)] } {
             set xdim [expr ($LX/2.0- $BP/2.0-4*$BP-$WInf + ($i-(($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0+(8*$BP)/(2*$xSize1) +1))*$xSize2)]
      } else { 
        if { $i > [expr (($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0)+(8*$BP)/(2*$xSize1)+(8*$BP)/(2*$xSize2)+$BP/$xSize3+ 2 + 1] && $i <= [expr $numXnode-(($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0-4*$BP/$xSize1) ] } { 
             set xdim [expr ($LX/2.0+ $BP/2.0 +$WInf+ ( $i-(($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0+(8*$BP)/(2*$xSize1)+(8*$BP)/(2*$xSize2)+$BP/$xSize3+2 +1))*$xSize2)]
      ### nodes xdim in zone 3
      } else {
        if { $i == $numXele/2 } {
             set xdim [ expr $LX/2- $BP/2]
      } else {
        if  { $i == $numXele/2 +3 } {
             set xdim [ expr $LX/2+ $BP/2+$WInf]
      } else {

             set xdim   [expr ($LX/2.0-$BP/2.0+($i-(($LX/2.0-8*$BP-$BP/2.0-$WInf)/$xSize0+ 8*$BP/(2*$xSize1)+8*$BP/(2*$xSize2)+1+1))*$xSize3) ]   
             }        
      }
      }
      }
      }
      }
      }
      }

      ### nodes ydim in zone 0
      if { $j <= [expr (($BY/2.0-8*$BP-$BP/2.0-$WInf)/$ySize0+1)] } {                                               
         set ydim [expr ($j-1)*$ySize0] 
        
      ### nodes ydim in zone 1
      } else {
        if { $j > [expr (($BY/2.0-8*$BP-$BP/2.0-$WInf)/$ySize0+1)] && $j <= [expr (($BY/2.0-8*$BP-$BP/2.0-$WInf)/$ySize0 + (8*$BP)/(2*$ySize1) + 1) ] } {
             set ydim [expr ($ySize1*($j-(($BY/2.0-8*$BP-$BP/2.0-$WInf)/$ySize0 + 1)) + $ySize0*($BY/2.0-8*$BP-$BP/2.0-$WInf)/$ySize0 ) ]             

      ### nodes ydim in zone 2 
      } else {
        if { $j > [expr (($BY/2.0-8*$BP-$BP/2.0-$WInf)/$ySize0+(8*$BP)/(2*$ySize1) + 1)] && $j <= [expr (($BY/2.0-8*$BP-$BP/2.0-$WInf)/$ySize0+(8*$BP)/(2*$ySize1) + (8*$BP)/(2*$ySize2) +1)] } {
             set ydim [expr ($BY/2.0- $BP/2.0-$WInf-4*$BP + ($j-(($BY/2.0-8*$BP-$BP/2.0-$WInf)/$ySize0+(8*$BP)/(2*$ySize1) +1))*$ySize2)]
      ### nodes ydim in zone 3
      } else {
         if { $j == $numYele} {
            set ydim [ expr $BY/2- $BP/2]
      } else {
         if { $j == [expr $numYele+1] || $j == [expr $numYele+2] } {
             set ydim   [expr ($BY/2.0-$BP/2.0 + ($j-$numYele)*$ySize3) ]    
      } else {
         if { $j == [expr $numYele+3] } {
             set ydim [expr $BY/2.0+$BP/2.0+$WInf ]
      } else {
         if { $j > [expr $numYele+3] && $j <= [expr $numYele+1+1+$eleNum_2+1] } {
             set ydim [expr $BY/2.0+$BP/2.0+$WInf+($j-($numYele+3))*$ySize2 ]
      } else {
         if { $j > [expr $numYele+1+1+$eleNum_2+1] && $j <= [expr $numYele+1+1+$eleNum_2+$eleNum_1+1] } {
             set ydim [expr $BY/2.0+$BP/2.0+$WInf+4.0*$BP+($j-($numYele+1+1+$eleNum_2+1))*$ySize1 ]
      } else {
         if { $j > [expr $numYele+1+1+$eleNum_2+$eleNum_1+1] } {
             set ydim [expr $BY/2.0+$BP/2.0+$WInf+8.0*$BP+($j-($numYele+1+1+$eleNum_2+$eleNum_1+1))*$ySize0 ]
     
      }
      }}
      }}}}}}
}
}
      if { $k <= [expr ($z2/$zSize2+1)] } {
          set   zdim [expr ($k-1)*$zSize2]
      } else {
         if { $k > [expr ($z2/$zSize2)+($z1/$zSize1)+1] } {
          set   zdim [expr $z2+$z1+($k-(($z2/$zSize2)+($z1/$zSize1)+1))*$zSize0]
      } else {
          set   zdim [expr $z2+($k-(($z2/$zSize2)+1))*$zSize1]
      }
      }
	   ## Top level
       node  [expr $ID+$m]    $xdim $ydim $zdim
	   ## Bottom level
	   node  [expr $ID_2+$m]  $xdim $ydim $zdim
	   
	   ## Top level
	   fix 	 [expr $ID+$m]    0 1 1 ; #User-defined --- release constraint in other directions if you want to apply motion in different directions.
	   ## Bottom level
	   fix 	 [expr $ID_2+$m]  1 1 1
	   ## Tie top level nodes to original base nodes
	   equalDOF [expr $ID+$m]  [expr $m*$numZnode+$k]   1 2 3
    }
    set m [expr ($m+1)]
  } 
}

# User-Defined --- Material properties below model base
set rockDen 25.0        ;# User-Defined  --- Rock Unit weight
set rockVs  400.0       ;# User-Defined  --- Rock shear wave velocity
set rockPoisson 0.25    ;# User-Defined  --- Rock Poisson's ratio

# Compressive wave (p-wave) --- In case you need to apply motion in vertical direction
set rockG [expr $rockVs*$rockVs*$rockDen]
set rockM [expr 2*$rockG*(1-$rockPoisson)/(1-2*$rockPoisson)]
set rockVp [expr pow($rockM/$rockDen,0.5)]


# Define Dashpot Material Elements
set size_average_X 2.50; #User-Defined --- Average element size in direction X
set size_average_Y 2.50; #User-Defined --- Average element size in direction Y
set m 0
for {set i 1} {$i <= $numXnode} {incr i 1} {
  for {set j 1} {$j <= $numYnode} {incr j 1} {

		uniaxialMaterial Viscous [expr 500000+$m] [expr $rockDen*$rockVs*$size_average_X*$size_average_Y] 1.0  ; # Note that we have assumed that 
		element zeroLength       [expr 600000+$m]  [expr $ID_2+$m]  [expr $ID+$m] -mat [expr 500000+$m]   -dir  1
		set m [expr ($m+1)]
      }
    } 
	
# load pattern for u-rock 
set FACT  1
set Gaccel 1000
timeSeries Path $Gaccel -fileTime Time.txt -filePath Velocity.txt  -factor $FACT

pattern Plain 300 1000 {
	set m 0
		for {set i 1} {$i <= $numXnode} {incr i 1} {
			for {set j 1} {$j <= $numYnode} {incr j 1} {
				load    [expr $ID+$m]    [expr $rockDen*$rockVs*$xSize1*$ySize1] 0.0 0.0
				set m [expr ($m+1)]
				}
			}

}

########################################################################
#### End of Compliant Base Boundary Condition
########################################################################

########################################
#### Define Seismic Analysis Algorithms
########################################

set compdT  0.005		; #User-Defined --- Time-step
set EQduration 29.50	; #User-Defined --- Duration of input earthquake motion

set gamma  0.6
integrator Newmark  $gamma  [expr pow($gamma+0.5, 2)/4.0]

#integrator HHT 0.5 
#numberer RCM
numberer Plain
constraints Penalty 1.0e18 1.0e18
test NormDispIncr 1e-2 50 1
algorithm KrylovNewton
system SparseSYM
#system Mumps
analysis VariableTransient

 for {set numIncr 1} {$numIncr <= [expr int($EQduration/$compdT)] } {incr numIncr 1} {
        puts "##### Seismic excitation step : $numIncr #####";
        analyze 1 $compdT [expr $compdT/100.0] $compdT  20
        }

set endT [clock seconds]
set ElapsedTime [expr $endT-$startT]
set ElapsedHours [expr int($ElapsedTime/3600.)]
set ElapsedMins [expr int($ElapsedTime/60.-$ElapsedHours*60.)]
set ElapsedSecs [expr int($ElapsedTime-$ElapsedHours*3600-$ElapsedMins*60.)]
puts "Execution time: $ElapsedHours hours and $ElapsedMins minutes and $ElapsedSecs seconds !!"

########################################
#### End of the TCL Code
########################################

wipe



